@@ -1,10 +1,17 @@
-# $Id: Preprocessor.pm,v 1.23 2001/07/25 15:28:24 rcaputo Exp $
+# $Id: Preprocessor.pm,v 1.29 2002/06/02 16:02:50 rcaputo Exp $
 
 package POE::Preprocessor;
 
 use strict;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.29 $ ))[1];
+
 use Carp qw(croak);
 use Filter::Util::Call;
+use Symbol qw(gensym);
+use File::Path qw(mkpath);
+use File::Spec;
 
 sub MAC_PARAMETERS () { 0 }
 sub MAC_CODE       () { 1 }
@@ -19,16 +26,32 @@ sub COND_FLAG   () { 0 }
 sub COND_LINE   () { 1 }
 sub COND_INDENT () { 2 }
 
+BEGIN {
+  if (defined $ENV{POE_PREPROC_DUMP}) {
+    eval "sub DUMP_EXPANDED () { 1 }";
+    eval "sub DUMP_BASE     () { '$ENV{POE_PREPROC_DUMP}' }";
+  }
+  else {
+    eval "sub DUMP_EXPANDED () { 0 }";
+    eval "sub DUMP_BASE     () { die }";
+  }
+};
+
 #sub DEBUG () { 1 }
 #sub DEBUG_INVOKE () { 1 }
 #sub DEBUG_DEFINE () { 1 }
 
+#sub WARN_DEFINE  () { 1 }
+
 BEGIN {
   defined &DEBUG        or eval 'sub DEBUG        () { 0 }'; # preprocessor
   defined &DEBUG_INVOKE or eval 'sub DEBUG_INVOKE () { 0 }'; # macro invocs
   defined &DEBUG_DEFINE or eval 'sub DEBUG_DEFINE () { 0 }'; # macro defines
+  defined &WARN_DEFINE  or eval 'sub WARN_DEFINE  () { 0 }'; # redefine warning
 };
 
+### Start of regexp optimizer.
+
 # text_trie_trie is virtually identical to code in Ilya Zakharevich's
 # Text::Trie::Trie function.  The minor differences involve hardcoding
 # the minimum substring length to 1 and sorting the output.
@@ -126,12 +149,60 @@ sub fix_exclude {
 my (%constants, %macros, %const_regexp, %macro);
 
 sub import {
-  # Outer closure to define a unique scope.
-  { my $macro_name = '';
-    my ($macro_line, $enum_index);
-    my ($package_name, $file_name, $line_number) = (caller)[0,1,2];
-    my $const_regexp_dirty = 0;
-    my $state = STATE_PLAIN;
+
+    my $self = shift;
+    my %args;
+    if(@_ > 1) {
+        %args = @_;
+    }
+
+    # Outer closure to define a unique scope.
+    { my $macro_name = '';
+      my ($macro_line, $enum_index);
+      my ($package_name, $file_name, $line_number) = (caller)[0,1,2];
+      my $const_regexp_dirty = 0;
+      my $state = STATE_PLAIN;
+      my $dump_file;
+
+      DUMP_EXPANDED and do {
+        my ($vol, $dir, $file) = File::Spec->splitpath($file_name);
+        my $dest_dir      = File::Spec->catdir(DUMP_BASE, $dir);
+        my $dest_filename = File::Spec->catfile($dest_dir, $file);
+
+        mkpath($dest_dir, 0, 0777);
+        $dump_file = gensym;
+        open $dump_file, ">$dest_filename"
+          or die "POE::Preprocessor could not create $dest_filename: $!";
+        print $dump_file "package $package_name;\n";
+      };
+
+    # The following block processes inheritance requests for
+    # macros/constants and enums.  added by sungo 09/2001
+    my @isas;
+
+    if($args{isa}) {
+        if(ref $args{isa} eq 'ARRAY') {
+            foreach my $isa (@{$args{isa}}) {
+                push @isas, $isa;
+            }
+        } else {
+            push @isas, $args{isa};
+        }
+        foreach my $isa (@isas) {
+            eval "use $isa";
+            croak "Unable to load $isa : $@" if $@;
+
+            foreach my $const (keys %{$constants{$isa}}) {
+                $constants{$package_name}->{$const} =
+                  $constants{$isa}->{$const};
+                $const_regexp_dirty = 1;
+            }
+
+            foreach my $macro (keys %{$macros{$isa}}) {
+                $macros{$package_name}->{$macro} = $macros{$isa}->{$macro};
+            }
+        }
+    }
 
     $conditional_stacks{$package_name} = [ ];
     $excluding_code{$package_name} = 0;
@@ -139,7 +210,7 @@ sub import {
     my $set_const = sub {
       my ($name, $value) = @_;
 
-      if (exists $constants{$package_name}->{$name}) {
+      if (WARN_DEFINE && exists $constants{$package_name}->{$name}) {
         warn "const $name redefined at $file_name line $line_number\n"
           unless $constants{$package_name}->{$name} eq $value;
       }
@@ -168,6 +239,7 @@ sub import {
                    $conditional_stacks{$package_name}->[0]->[COND_LINE] . "\n"
                  );
             }
+            DUMP_EXPANDED and close $dump_file;
             return $status;
           }
 
@@ -202,6 +274,10 @@ sub import {
                 DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
               }
 
+              DUMP_EXPANDED and do {
+                print $dump_file $_;
+                close $dump_file if $status <= 0;
+              };
               return $status;
             }
 
@@ -213,6 +289,10 @@ sub import {
 
               unless ($state & STATE_MACRO_DEF) {
                 DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
+                DUMP_EXPANDED and do {
+                  print $dump_file $_;
+                  close $dump_file if $status <= 0;
+                };
                 return $status;
               }
             }
@@ -223,6 +303,10 @@ sub import {
                 die( "else { # include ... without if or unless " .
                      "at $file_name line $line_number\n"
                    );
+                DUMP_EXPANDED and do {
+                  print $dump_file $_;
+                  close $dump_file;
+                };
                 return -1;
               }
 
@@ -233,6 +317,11 @@ sub import {
 
               unless ($state & STATE_MACRO_DEF) {
                 DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
+
+                DUMP_EXPANDED and do {
+                  print $dump_file $_;
+                  close $dump_file if $status <= 0;
+                };
                 return $status;
               }
             }
@@ -261,6 +350,10 @@ sub import {
                 DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
               }
 
+              DUMP_EXPANDED and do {
+                print $dump_file $_;
+                close $dump_file if $status <= 0;
+              };
               return $status;
             }
 
@@ -270,6 +363,10 @@ sub import {
                 die( "Include elsif without include if or unless " .
                      "at $file_name line $line_number\n"
                    );
+                DUMP_EXPANDED and do {
+                  print $dump_file $_;
+                  close $dump_file;
+                };
                 return -1;
               }
 
@@ -295,6 +392,10 @@ sub import {
                 DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
               }
 
+              DUMP_EXPANDED and do {
+                print $dump_file $_;
+                close $dump_file if $status <= 0;
+              };
               return $status;
             }
           }
@@ -310,6 +411,10 @@ sub import {
 
             unless ($state & STATE_MACRO_DEF) {
               DEBUG and warn sprintf "%4d C: %s", $line_number, $_;
+              DUMP_EXPANDED and do {
+                print $dump_file $_;
+                close $dump_file if $status <= 0;
+              };
               return $status;
             }
           }
@@ -336,7 +441,9 @@ sub import {
               $macro{$package_name}->[MAC_CODE] =~ s/^\s*//;
               $macro{$package_name}->[MAC_CODE] =~ s/\s*$//;
 
-              if (exists $macros{$package_name}->{$macro_name}) {
+              if ( WARN_DEFINE &&
+                   exists $macros{$package_name}->{$macro_name}
+                 ) {
                 warn( "macro $macro_name redefined at ",
                       "$file_name line $line_number\n"
                     )
@@ -360,6 +467,10 @@ sub import {
             $_ = "# mac 4: $_";
             DEBUG and warn sprintf "%4d M: %s", $line_number, $_;
 
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file if $status <= 0;
+            };
             return $status;
           }
 
@@ -368,11 +479,21 @@ sub import {
           ### expense of preprocessing data and documentation.
           if (/^__(END|DATA)__\s*$/) {
             $_ = "# $_";
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file;
+            };
             return 0;
           }
 
           ### We're done if we're excluding code.
-          return $status if $excluding_code{$package_name};
+          if ($excluding_code{$package_name}) {
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file if $status <= 0;
+            };
+            return $status;
+          }
 
           ### Define an enum.
           if (/^enum(?:\s+(\d+|\+))?\s+(.*?)\s*$/) {
@@ -393,6 +514,10 @@ sub import {
 
             DEBUG and warn sprintf "%4d E: %s", $line_number, $_;
 
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file if $status <= 0;
+            };
             return $status;
           }
 
@@ -401,6 +526,11 @@ sub import {
             &{$set_const}($1, $2);
             $_ = "# $_";
             DEBUG and warn sprintf "%4d E: %s", $line_number, $_;
+
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file if $status <= 0;
+            };
             return $status;
           }
 
@@ -428,6 +558,11 @@ sub import {
 
             $_ = "# $temp_line";
             DEBUG and warn sprintf "%4d D: %s", $line_number, $_;
+
+            DUMP_EXPANDED and do {
+              print $dump_file $_;
+              close $dump_file if $status <= 0;
+            };
             return $status;
           }
 
@@ -456,6 +591,11 @@ sub import {
                       scalar(@mac_params),
                       ") at $file_name line $line_number\n"
                     );
+
+                DUMP_EXPANDED and do {
+                  print $dump_file $_;
+                  close $dump_file if $status <= 0;
+                };
                 return $status;
               }
 
@@ -492,9 +632,9 @@ sub import {
               $substitutions++;
             }
             else {
-              warn( "macro $name has not been defined ",
-                    "at $file_name line $line_number\n"
-                  );
+              die( "macro $name has not been defined ",
+                   "at $file_name line $line_number\n"
+                 );
               last;
             }
           }
@@ -529,7 +669,11 @@ sub import {
             }
           }
 
-          $status;
+          DUMP_EXPANDED and do {
+            print $dump_file $_;
+            close $dump_file if $status <= 0;
+          };
+          return $status;
         }
       );
   }
@@ -556,6 +700,8 @@ POE::Preprocessor - a macro/const/enum preprocessor
 
   use POE::Preprocessor;
 
+  # use POE::Preprocessor ( isa => 'POE::SomeModule' );
+
   macro max (one,two) {
     ((one) > (two) ? (one) : (two))
   }
@@ -676,6 +822,19 @@ previous example.
 Conditional includes are experimental pending a decision on how useful
 they are.
 
+=head1 IMPORTING MACROS/CONSTANTS
+
+    use POE::Preprocessor ( isa => 'POE::SomeModule' );
+
+This method of calling Preprocessor causes the macros and constants of 
+C<POE::SomeModule> to be imported for use in the current namespace. 
+These macros and constants can be overriden simply by defining items 
+in the current namespace of the same name.
+
+Note: if the macros in C<POE::SomeModule> require additional perl 
+modules, any code which imports these macros will need to C<use> 
+those modules as well.
+
 =head1 DEBUGGING
 
 POE::Preprocessor has three debugging constants which may be defined
@@ -694,6 +853,42 @@ To see macro, constant, and enum definitions:
 
   sub POE::Preprocessor::DEBUG_DEFINE () { 1 }
 
+To see warnings when a macro or constant is redefined:
+
+  sub POE::Preprocessor::WARN_DEFINE () { 1 }
+
+=head1 ENVIRONMENT
+
+Setting the POE_PREPROC_DUMP environment variable causes
+POE::Preprocessor to write new copies of the modules it expands.
+POE_PREPROC_DUMP should contain the name of a base directory.  It
+need not exist.  All the expanded files will be written under the
+directory in POE_PREPROC_DUMP.
+
+Note: Some modules such as POE::Kernel alter the macros they load at
+compile time.  Versions of these modules created by POE_PREPROC_DUMP
+may only be used in the same situation they were created in.  For
+instance, a version of POE::Kernel created with Gtk support macros may
+only be used in Gtk programs from that point on.
+
+Note: Because POE::Preprocessor can only dump source code from the
+point it it is used in a file, it should be the first statement after
+C<package Foo;> in a module.  Otherwise code before it will be missing
+from the resulting expanded file.  POE::Preprocessor will create a new
+C<package Foo;> line to replace the one that it could not see.
+
+This is good:
+
+  package Foo;  # Not seen but simulated in the expanded version.
+  use POE::Preprocessor;
+  use Carp;     # Seen and included in the expanded version.
+
+This is bad:
+
+  package Foo;  # Not seen but simulated in the expanded version.
+  use Carp;     # Not seen and OMITTED FROM the expanded version.
+  use POE::Preprocessor;
+
 =head1 BUGS
 
 Source filters are line-based, and so is the macro language.  The only
@@ -712,6 +907,9 @@ The regexp builder makes silly subexpressions like /(?:|m)/.  That
 could be done better as /m?/ or /(?:jklm)?/ if the literal is longer
 than a single character.
 
+The "# include" directives are not compatible with the
+POE_PREPROC_DUMP environment variable.
+
 =head1 SEE ALSO
 
 The regexp optimizer is based on code in Ilya Zakharevich's
@@ -719,8 +917,9 @@ Text::Trie.
 
 =head1 AUTHOR & COPYRIGHT
 
-POE::Preprocessor is Copyright 2000 Rocco Caputo.  All rights
-reserved.  POE::Preprocessor is free software; you may redistribute it
-and/or modify it under the same terms as Perl itself.
+POE::Preprocessor is Copyright 2000 Rocco Caputo.  Some parts are 
+Copyright 2001 Matt Cashner. All rights reserved.  POE::Preprocessor 
+is free software; you may redistribute it and/or modify it under 
+the same terms as Perl itself.
 
 =cut
