@@ -1,26 +1,37 @@
-# $Id: FollowTail.pm,v 1.25 2001/08/11 22:40:05 rcaputo Exp $
+# $Id: FollowTail.pm,v 1.34 2002/05/28 22:46:06 rcaputo Exp $
 
 package POE::Wheel::FollowTail;
 
 use strict;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.34 $ ))[1];
+
 use Carp;
+use Symbol;
 use POSIX qw(SEEK_SET SEEK_CUR SEEK_END);
-use POE qw(Wheel);
+use POE qw(Wheel Driver::SysRW Filter::Line);
 
 sub CRIMSON_SCOPE_HACK ($) { 0 }
 
-sub SELF_HANDLE      () { 0 }
-sub SELF_DRIVER      () { 1 }
-sub SELF_FILTER      () { 2 }
-sub SELF_INTERVAL    () { 3 }
-sub SELF_EVENT_INPUT () { 4 }
-sub SELF_EVENT_ERROR () { 5 }
-sub SELF_UNIQUE_ID   () { 6 }
-sub SELF_STATE_READ  () { 7 }
-sub SELF_STATE_WAKE  () { 8 }
+sub SELF_HANDLE      () {  0 }
+sub SELF_FILENAME    () {  1 }
+sub SELF_DRIVER      () {  2 }
+sub SELF_FILTER      () {  3 }
+sub SELF_INTERVAL    () {  4 }
+sub SELF_EVENT_INPUT () {  5 }
+sub SELF_EVENT_ERROR () {  6 }
+sub SELF_EVENT_RESET () {  7 }
+sub SELF_UNIQUE_ID   () {  8 }
+sub SELF_STATE_READ  () {  9 }
+sub SELF_STATE_WAKE  () { 10 }
+sub SELF_LAST_STAT   () { 11 }
 
 # Turn on tracing.  A lot of debugging occurred just after 0.11.
-sub TRACE () { 0 }
+sub TRACE_RESET        () { 0 }
+sub TRACE_STAT         () { 0 }
+sub TRACE_STAT_VERBOSE () { 0 }
+sub TRACE_POLL         () { 0 }
 
 # Tk doesn't provide a SEEK method, as of 800.022
 BEGIN {
@@ -50,34 +61,45 @@ sub new {
 
   # STATE-EVENT
   if (exists $params{InputState}) {
+    carp "InputState is deprecated.  Use InputEvent";
     if (exists $params{InputEvent}) {
-      carp "InputEvent takes precedence over deprecated InputState";
       delete $params{InputState};
     }
     else {
-      # deprecation warning goes here
       $params{InputEvent} = delete $params{InputState};
     }
   }
 
   # STATE-EVENT
   if (exists $params{ErrorState}) {
+    carp "ErrorState is deprecated.  Use ErrorEvent";
     if (exists $params{ErrorEvent}) {
-      carp "ErrorEvent takes precedence over deprecated ErrorState";
       delete $params{ErrorState};
     }
     else {
-      # deprecation warning goes here
       $params{ErrorEvent} = delete $params{ErrorState};
     }
   }
 
-  croak "Handle required"     unless defined $params{Handle};
-  croak "Driver required"     unless defined $params{Driver};
-  croak "Filter required"     unless defined $params{Filter};
+  croak "Handle or Filename required, but not both"
+    unless $params{Handle} xor defined $params{Filename};
+
+  my $driver = delete $params{Driver};
+  $driver = POE::Driver::SysRW->new() unless defined $driver;
+
+  my $filter = delete $params{Filter};
+  $filter = POE::Filter::Line->new() unless defined $filter;
+
   croak "InputEvent required" unless defined $params{InputEvent};
 
-  my ($handle, $driver, $filter) = @params{ qw(Handle Driver Filter) };
+  my ($handle, $filename) = @params{ qw(Handle Filename) };
+
+  my @start_stat;
+  if (defined $filename) {
+    $handle = gensym();
+    open $handle, "<$filename" or croak "can't open $filename: $!";
+    @start_stat = stat($filename);
+  }
 
   my $poll_interval = ( (defined $params{PollInterval})
                         ? $params{PollInterval}
@@ -91,14 +113,17 @@ sub new {
   $seek_back = 0 if $seek_back < 0;
 
   my $self = bless [ $handle,                          # SELF_HANDLE
+                     $filename,                        # SELF_FILENAME
                      $driver,                          # SELF_DRIVER
                      $filter,                          # SELF_FILTER
                      $poll_interval,                   # SELF_INTERVAL
                      delete $params{InputEvent},       # SELF_EVENT_INPUT
                      delete $params{ErrorEvent},       # SELF_EVENT_ERROR
+                     delete $params{ResetEvent},       # SELF_EVENT_RESET
                      &POE::Wheel::allocate_wheel_id(), # SELF_UNIQUE_ID
                      undef,                            # SELF_STATE_READ
                      undef,                            # SELF_STATE_WAKE
+                     \@start_stat,                     # SELF_LAST_STAT
                   ], $type;
 
   $self->_define_states();
@@ -153,15 +178,20 @@ sub _define_states {
   my $driver        = $self->[SELF_DRIVER];
   my $event_input   = \$self->[SELF_EVENT_INPUT];
   my $event_error   = \$self->[SELF_EVENT_ERROR];
-  my $state_wake    = $self->[SELF_STATE_WAKE] = $self . ' alarm';
-  my $state_read    = $self->[SELF_STATE_READ] = $self . ' select read';
-  my $poll_interval = $self->[SELF_INTERVAL];
-  my $handle        = $self->[SELF_HANDLE];
+  my $event_reset   = \$self->[SELF_EVENT_RESET];
   my $unique_id     = $self->[SELF_UNIQUE_ID];
+  my $state_wake    = $self->[SELF_STATE_WAKE] =
+    ref($self) . "($unique_id) -> alarm";
+  my $state_read    = $self->[SELF_STATE_READ] =
+    ref($self) . "($unique_id) -> select read";
+  my $poll_interval = $self->[SELF_INTERVAL];
+  my $filename      = $self->[SELF_FILENAME];
+  my $last_stat     = $self->[SELF_LAST_STAT];
+  my $handle        = $self->[SELF_HANDLE];
 
   # Define the read state.
 
-  TRACE and do { warn $state_read; };
+  TRACE_POLL and do { warn $state_read; };
 
   $poe_kernel->state
     ( $state_read,
@@ -171,38 +201,95 @@ sub _define_states {
         0 && CRIMSON_SCOPE_HACK('<');
 
         # The actual code starts here.
-        my ($k, $ses, $hdl) = @_[KERNEL, SESSION, ARG0];
+        my ($k, $ses) = @_[KERNEL, SESSION];
 
-        $k->select_read($hdl);
+        eval {
+          if (defined $filename) {
+            my @new_stat = stat($filename);
 
-        eval { sysseek($hdl, 0, SEEK_CUR); };
+            TRACE_STAT_VERBOSE and do {
+              my @test_new = @new_stat;   splice(@test_new, 8, 1, "(removed)");
+              my @test_old = @$last_stat; splice(@test_old, 8, 1, "(removed)");
+              warn "=== @test_new" if "@test_new" ne "@test_old";
+            };
+
+            if (@new_stat) {
+              if ($new_stat[7] < $last_stat->[7]) {
+                $$event_reset and $k->call( $ses,
+                                            $$event_reset, $unique_id
+                                          );
+                $last_stat->[7] = $new_stat[7];
+              }
+
+              if ( $new_stat[1] != $last_stat->[1] or # inode's number
+                   $new_stat[0] != $last_stat->[0] or # inode's device
+                   $new_stat[6] != $last_stat->[6] or # device type
+                   $new_stat[3] != $last_stat->[3]    # number of links
+                 ) {
+
+                TRACE_STAT and do {
+                  warn "inode $new_stat[1] != old $last_stat->[1]\n"
+                    if $new_stat[1] != $last_stat->[1];
+                  warn "inode device $new_stat[0] != old $last_stat->[0]\n"
+                    if $new_stat[0] != $last_stat->[0];
+                  warn "device type $new_stat[6] != old $last_stat->[6]\n"
+                    if $new_stat[6] != $last_stat->[6];
+                  warn "number of links $new_stat[3] != old $last_stat->[3]\n"
+                    if $new_stat[3] != $last_stat->[3];
+                  warn "file size $new_stat[7] < old $last_stat->[7]\n"
+                    if $new_stat[7] < $last_stat->[7];
+                };
+
+                @$last_stat = @new_stat;
+
+                close $handle;
+                unless (open $handle, "<$filename") {
+                  $$event_error and
+                    $k->call( $ses, $$event_error, 'reopen',
+                              ($!+0), $!, $unique_id
+                            );
+                }
+              }
+              else {
+                sysseek($handle, 0, SEEK_CUR);
+              }
+            }
+          }
+          else {
+            sysseek($handle, 0, SEEK_CUR);
+          }
+        };
         $! = 0;
 
-        TRACE and do { warn time . " read ok\n"; };
+        TRACE_POLL and do { warn time . " read ok\n"; };
 
-        if (defined(my $raw_input = $driver->get($hdl))) {
-          TRACE and do { warn time . " raw input\n"; };
-          foreach my $cooked_input (@{$filter->get($raw_input)}) {
-            TRACE and do { warn time . " cooked input\n"; };
-            $k->call($ses, $$event_input, $cooked_input, $unique_id);
+        if (defined(my $raw_input = $driver->get($handle))) {
+          if (@$raw_input) {
+            TRACE_POLL and do { warn time . " raw input\n"; };
+            foreach my $cooked_input (@{$filter->get($raw_input)}) {
+              TRACE_POLL and do { warn time . " cooked input\n"; };
+              $k->call($ses, $$event_input, $cooked_input, $unique_id);
+            }
           }
         }
+        else {
+          TRACE_POLL and do { warn time . " set delay\n"; };
+          $k->delay($state_wake, $poll_interval);
+          $k->select_read($handle);
+        }
 
         if ($!) {
-          TRACE and do { warn time . " error: $!\n"; };
+          TRACE_POLL and do { warn time . " error: $!\n"; };
           $$event_error and
             $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
         }
-
-        TRACE and do { warn time . " set delay\n"; };
-        $k->delay($state_wake, $poll_interval);
       }
     );
 
   # Define the alarm state that periodically wakes the wheel and
   # retries to read from the file.
 
-  TRACE and do { warn $state_wake; };
+  TRACE_POLL and do { warn $state_wake; };
 
   $poe_kernel->state
     ( $state_wake,
@@ -212,7 +299,7 @@ sub _define_states {
                                         # subroutine starts here
         my $k = $_[KERNEL];
 
-        TRACE and do { warn time . " wake up and select the handle\n"; };
+        TRACE_POLL and do { warn time . " wake up and select the handle\n"; };
 
         $k->select_read($handle, $state_read);
       }
@@ -230,7 +317,7 @@ sub event {
 
     # STATE-EVENT
     if ($name =~ /^(.*?)State$/) {
-      # deprecation warning goes here
+      carp "$name is deprecated.  Use $1Event";
       $name = $1 . 'Event';
     }
 
@@ -245,6 +332,9 @@ sub event {
     elsif ($name eq 'ErrorEvent') {
       $self->[SELF_EVENT_ERROR] = $event;
     }
+    elsif ($name eq 'ResetEvent') {
+      $self->[SELF_EVENT_RESET] = $event;
+    }
     else {
       carp "ignoring unknown FollowTail parameter '$name'";
     }
@@ -291,12 +381,24 @@ POE::Wheel::FollowTail - follow the tail of an ever-growing file
 =head1 SYNOPSIS
 
   $wheel = POE::Wheel::FollowTail->new(
-    Handle       => $file_handle,                  # File to tail
+    Filename     => $file_name,                    # File to tail
     Driver       => POE::Driver::Something->new(), # How to read it
     Filter       => POE::Filter::Something->new(), # How to parse it
     PollInterval => 1,                  # How often to check it
     InputEvent   => $input_event_name,  # Event to emit upon input
     ErrorEvent   => $error_event_name,  # Event to emit upon error
+    ResetEvent   => $reset_event_name,  # Event to emit on file reset
+    SeekBack     => $offset,            # How far from EOF to start
+  );
+
+  $wheel = POE::Wheel::FollowTail->new(
+    Handle       => $open_file_handle,             # File to tail
+    Driver       => POE::Driver::Something->new(), # How to read it
+    Filter       => POE::Filter::Something->new(), # How to parse it
+    PollInterval => 1,                  # How often to check it
+    InputEvent   => $input_event_name,  # Event to emit upon input
+    ErrorEvent   => $error_event_name,  # Event to emit upon error
+    # No reset event available.
     SeekBack     => $offset,            # How far from EOF to start
   );
 
@@ -316,7 +418,8 @@ This is a read-only wheel so it does not include a put() method.
 
 event() is covered in the POE::Wheel manpage.
 
-FollowTail's event types are C<InputEvent> and C<ErrorEvent>.
+FollowTail's event types are C<InputEvent>, C<ResetEvent>, and
+C<ErrorEvent>.
 
 =item ID
 
@@ -330,6 +433,19 @@ match events with the wheels which generated them.
 
 =over 2
 
+=item Driver
+
+Driver is a POE::Driver subclass that is used to read from and write
+to FollowTail's filehandle.  It encapsulates the low-level I/O
+operations needed to access a file so in theory FollowTail never needs
+to know about them.
+
+=item Filter
+
+Filter is a POE::Filter subclass that is used to parse input from the
+tailed file.  It encapsulates the lowest level of a protocol so that
+in theory FollowTail never needs to know about file formats.
+
 =item PollInterval
 
 PollInterval is the amount of time, in seconds, the wheel will wait
@@ -351,6 +467,24 @@ When SeekBack is used, the wheel assumes that records have already
 been framed, and the seek position is the beginning of one.  It will
 return everything it reads up until EOF.
 
+=item Handle
+
+=item Filename
+
+Either the Handle or Filename constructor parameter is required, but
+you cannot supply both.
+
+FollowTail can watch a file or device that's already open.  Give it
+the open filehandle with its Handle parameter.
+
+FollowTail can watch a file by name, given as the Filename parameter.
+
+This wheel can detect files that have been "reset".  That is, it can
+tell when log files have been restarted due to a rotation or purge.
+For FollowTail to do this, though, it requires a Filename parameter.
+This is so FollowTail can reopen the file after it has reset.  See
+C<ResetEvent> elsewhere in this document.
+
 =item InputEvent
 
 InputEvent contains the name of an event which is emitted for every
@@ -365,6 +499,16 @@ A sample InputEvent event handler:
     print "Wheel $wheel_id received input: $input\n";
   }
 
+=item ResetEvent
+
+ResetEvent contains the name of an event that's emitted every time a
+file is reset.
+
+It's only available when watching files by name.  This is because
+FollowTail must reopen the file after it has been reset.
+
+C<ARG0> contains the FollowTail wheel's unique ID.
+
 =item ErrorEvent
 
 ErrorEvent contains the event which is emitted whenever an error
@@ -400,6 +544,16 @@ the entire POE distribution.
 
 This wheel can't tail pipes and consoles on some systems.
 
+Because this wheel is cooperatively multitasked, it may lose records
+just prior to a file reset.  For a more robust way to watch files,
+consider using POE::Wheel::Run and your operating system's native
+"tail" utility instead.
+
+  $heap->{tail} = POE::Wheel::Run->new
+    ( Program     => [ "/usr/bin/tail", "-f", $file_name ],
+      StdoutEvent => "log_record",
+    );
+
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
