@@ -1,8 +1,12 @@
-# $Id: ListenAccept.pm,v 1.21 2001/08/11 22:40:05 rcaputo Exp $
+# $Id: ListenAccept.pm,v 1.25 2002/05/10 01:39:04 rcaputo Exp $
 
 package POE::Wheel::ListenAccept;
 
 use strict;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.25 $ ))[1];
+
 use Carp;
 use Symbol;
 
@@ -30,24 +34,22 @@ sub new {
 
   # STATE-EVENT
   if (exists $params{AcceptState}) {
+    carp "AcceptState is deprecated.  Use AcceptEvent";
     if (exists $params{AcceptEvent}) {
-      carp "AcceptEvent takes precedence over deprecated AcceptState";
       delete $params{AcceptState};
     }
     else {
-      # deprecation warning goes here
       $params{AcceptEvent} = delete $params{AcceptState};
     }
   }
 
   # STATE-EVENT
   if (exists $params{ErrorState}) {
+    carp "ErrorState is deprecated.  Use ErrorEvent";
     if (exists $params{ErrorEvent}) {
-      carp "ErrorEvent takes precedence over deprecated ErrorState";
       delete $params{ErrorState};
     }
     else {
-      # deprecation warning goes here
       $params{ErrorEvent} = delete $params{ErrorState};
     }
   }
@@ -79,7 +81,7 @@ sub event {
 
     # STATE-EVENT
     if ($name =~ /^(.*?)State$/) {
-      # deprecation warning goes here
+      carp "$name is deprecated.  Use $1Event";
       $name = $1 . 'Event';
     }
 
@@ -113,7 +115,7 @@ sub _define_accept_state {
   my $unique_id    = $self->[SELF_UNIQUE_ID];
                                         # register the select-read handler
   $poe_kernel->state
-    ( $self->[SELF_STATE_ACCEPT] =  $self . ' select read',
+    ( $self->[SELF_STATE_ACCEPT] =  ref($self) . "($unique_id) -> select read",
       sub {
         # prevents SEGV
         0 && CRIMSON_SCOPE_HACK('<');
