@@ -1,14 +1,21 @@
-# $Id: Run.pm,v 1.19 2001/08/29 19:44:45 rcaputo Exp $
+# $Id: Run.pm,v 1.32 2002/05/28 22:49:15 rcaputo Exp $
 
 package POE::Wheel::Run;
 
 use strict;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.32 $ ))[1];
+
 use Carp;
 use POSIX;  # termios stuff
 
-use POE qw(Wheel Pipe::TwoWay Pipe::OneWay Driver::SysRW);
+use POE qw( Wheel Pipe::TwoWay Pipe::OneWay Driver::SysRW Filter::Line );
 
 BEGIN {
+  die "$^O does not support fork()\n" if $^O eq 'MacOS';
+  die "$^O does not fully support fork/exec\n" if $^O eq 'MSWin32';
+
   local $SIG{'__DIE__'} = 'DEFAULT';
   eval    { require IO::Pty; };
   if ($@) { eval 'sub PTY_AVAILABLE () { 0 }';  }
@@ -32,11 +39,15 @@ BEGIN {
     eval 'sub CIBAUD () { undef; }';
   }
 
-  if (eval '&IO::Tty::Constant::TIOCSWINSZ') {
+  if ( eval '&IO::Tty::Constant::TIOCSWINSZ' and
+       eval '&IO::Tty::Constant::TIOCGWINSZ'
+     ) {
     *TIOCSWINSZ = *IO::Tty::Constant::TIOCSWINSZ;
+    *TIOCGWINSZ = *IO::Tty::Constant::TIOCGWINSZ;
   }
   else {
     eval 'sub TIOCSWINSZ () { undef; }';
+    eval 'sub TIOCGWINSZ () { undef; }';
   }
 };
 
@@ -44,25 +55,27 @@ BEGIN {
 sub UNIQUE_ID     () {  0 }
 sub DRIVER        () {  1 }
 sub ERROR_EVENT   () {  2 }
-sub PROGRAM       () {  3 }
-sub CHILD_PID     () {  4 }
-sub CONDUIT_TYPE  () {  5 }
+sub CLOSE_EVENT   () {  3 }
+sub PROGRAM       () {  4 }
+sub CHILD_PID     () {  5 }
+sub CONDUIT_TYPE  () {  6 }
+sub IS_ACTIVE     () {  7 }
 
-sub HANDLE_STDIN  () {  6 }
-sub FILTER_STDIN  () {  7 }
-sub EVENT_STDIN   () {  8 }
-sub STATE_STDIN   () {  9 }
-sub OCTETS_STDIN  () { 10 }
+sub HANDLE_STDIN  () {  8 }
+sub FILTER_STDIN  () {  9 }
+sub EVENT_STDIN   () { 10 }
+sub STATE_STDIN   () { 11 }
+sub OCTETS_STDIN  () { 12 }
 
-sub HANDLE_STDOUT () { 11 }
-sub FILTER_STDOUT () { 12 }
-sub EVENT_STDOUT  () { 13 }
-sub STATE_STDOUT  () { 14 }
+sub HANDLE_STDOUT () { 13 }
+sub FILTER_STDOUT () { 14 }
+sub EVENT_STDOUT  () { 15 }
+sub STATE_STDOUT  () { 16 }
 
-sub HANDLE_STDERR () { 15 }
-sub FILTER_STDERR () { 16 }
-sub EVENT_STDERR  () { 17 }
-sub STATE_STDERR  () { 18 }
+sub HANDLE_STDERR () { 17 }
+sub FILTER_STDERR () { 18 }
+sub EVENT_STDERR  () { 19 }
+sub STATE_STDERR  () { 20 }
 
 # Used to work around a bug in older perl versions.
 sub CRIMSON_SCOPE_HACK ($) { 0 }
@@ -108,6 +121,9 @@ sub new {
           );
 
   my $all_filter    = delete $params{Filter};
+  $all_filter = POE::Filter::Line->new(Literal => "\n")
+    unless defined $all_filter;
+
   my $stdin_filter  = delete $params{StdinFilter};
   my $stdout_filter = delete $params{StdoutFilter};
   my $stderr_filter = delete $params{StderrFilter};
@@ -132,7 +148,8 @@ sub new {
   croak "$type needs either Filter or StderrFilter"
     if defined($stderr_event) and not defined($stderr_filter);
 
-  my $error_event   = delete $params{ErrorEvent};
+  my $error_event = delete $params{ErrorEvent};
+  my $close_event  = delete $params{CloseEvent};
 
   # Make sure the user didn't pass in parameters we're not aware of.
   if (scalar keys %params) {
@@ -205,8 +222,8 @@ sub new {
         ioctl( $stdin_read, TIOCSCTTY, 0 );
       }
 
-      # Put the pty conduit into "raw" or "cbreak" mode, per APITUE
-      # 19.4 and 11.10.
+      # Put the pty conduit (slave side) into "raw" or "cbreak" mode,
+      # per APITUE 19.4 and 11.10.
       my $tio = POSIX::Termios->new();
       $tio->getattr(fileno($stdin_read));
       my $lflag = $tio->getlflag;
@@ -222,6 +239,19 @@ sub new {
       $oflag &= ~(OPOST);
       $tio->setoflag($oflag);
       $tio->setattr(fileno($stdin_read), TCSANOW);
+
+      # Set the pty conduit (slave side) window size to our window
+      # size.  APITUE 19.4 and 19.5.
+      if (defined TIOCGWINSZ) {
+        if (-t STDIN) {
+          my $window_size = '';
+          ioctl( STDIN, TIOCGWINSZ, $window_size ) or die $!;
+          ioctl( $stdin_read, TIOCSWINSZ, $window_size ) or die $!;
+        }
+        else {
+          carp "STDIN is not a terminal.  Can't set slave pty's window size";
+        }
+      }
     }
 
     # -><- How to pass events to the parent process?  Maybe over a
@@ -282,9 +312,9 @@ sub new {
     select STDOUT;  $| = 1;
 
     # Tell the parent that the stdio has been set up.
-    close $sem_pipe_read;
+    close $sem_pipe_read unless $^O eq 'MSWin32';
     print $sem_pipe_write "go\n";
-    close $sem_pipe_write;
+    close $sem_pipe_write unless $^O eq 'MSWin32';
 
     # Exec the program depending on its form.
     if (ref($program) eq 'ARRAY') {
@@ -313,13 +343,19 @@ sub new {
   close $stdout_write if defined $stdout_write;
   close $stderr_write if defined $stderr_write;
 
+  my $handle_count = 0;
+  $handle_count++ if defined $stdout_read;
+  $handle_count++ if defined $stderr_read;
+
   my $self = bless
     [ &POE::Wheel::allocate_wheel_id(),  # UNIQUE_ID
       POE::Driver::SysRW->new(),         # DRIVER
       $error_event,   # ERROR_EVENT
+      $close_event,   # CLOSE_EVENT
       $program,       # PROGRAM
       $pid,           # CHILD_PID
       $conduit,       # CONDUIT_TYPE
+      $handle_count,  # IS_ACTIVE
       # STDIN
       $stdin_write,   # HANDLE_STDIN
       $stdin_filter,  # FILTER_STDIN
@@ -339,9 +375,9 @@ sub new {
     ], $type;
 
   # Wait here while the child sets itself up.
-  close $sem_pipe_write;
   <$sem_pipe_read>;
   close $sem_pipe_read;
+  close $sem_pipe_write;
 
   $self->_define_stdin_flusher();
   $self->_define_stdout_reader() if defined $stdout_event;
@@ -362,8 +398,10 @@ sub _define_stdin_flusher {
   my $unique_id     = $self->[UNIQUE_ID];
   my $driver        = $self->[DRIVER];
   my $error_event   = \$self->[ERROR_EVENT];
+  my $close_event   = \$self->[CLOSE_EVENT];
   my $stdin_filter  = $self->[FILTER_STDIN];
   my $stdin_event   = \$self->[EVENT_STDIN];
+  my $is_active     = \$self->[IS_ACTIVE];
 
   # Read/write members.  These are done by reference, to avoid pushing
   # $self into the anonymous sub.  Extra copies of $self are bad and
@@ -372,7 +410,7 @@ sub _define_stdin_flusher {
 
   # Register the select-write handler.
   $poe_kernel->state
-    ( $self->[STATE_STDIN] = $self . ' select stdin',
+    ( $self->[STATE_STDIN] = ref($self) . "($unique_id) -> select stdin",
       sub {                             # prevents SEGV
         0 && CRIMSON_SCOPE_HACK('<');
                                         # subroutine starts here
@@ -383,7 +421,7 @@ sub _define_stdin_flusher {
         # When you can't write, nothing else matters.
         if ($!) {
           $$error_event && $k->call( $me, $$error_event,
-                                     'write', ($!+0), $!, $unique_id
+                                     'write', ($!+0), $!, $unique_id, "STDIN"
                                    );
           $k->select_write($handle);
         }
@@ -425,11 +463,13 @@ sub _define_stdout_reader {
     my $unique_id     = $self->[UNIQUE_ID];
     my $driver        = $self->[DRIVER];
     my $error_event   = \$self->[ERROR_EVENT];
+    my $close_event   = \$self->[CLOSE_EVENT];
     my $stdout_filter = $self->[FILTER_STDOUT];
     my $stdout_event  = \$self->[EVENT_STDOUT];
+    my $is_active     = \$self->[IS_ACTIVE];
 
     $poe_kernel->state
-      ( $self->[STATE_STDOUT] = $self . ' select stdout',
+      ( $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
         sub {
           # prevents SEGV
           0 && CRIMSON_SCOPE_HACK('<');
@@ -443,7 +483,13 @@ sub _define_stdout_reader {
           }
           else {
             $$error_event and
-              $k->call( $me, $$error_event, 'read', ($!+0), $!, $unique_id );
+              $k->call( $me, $$error_event,
+                        'read', ($!+0), $!, $unique_id, 'STDOUT'
+                      );
+            unless (--$$is_active) {
+              $k->call( $me, $$close_event, $unique_id )
+                if defined $$close_event;
+            }
             $k->select_read($handle);
           }
         }
@@ -474,11 +520,13 @@ sub _define_stderr_reader {
     my $unique_id     = $self->[UNIQUE_ID];
     my $driver        = $self->[DRIVER];
     my $error_event   = \$self->[ERROR_EVENT];
+    my $close_event   = \$self->[CLOSE_EVENT];
     my $stderr_filter = $self->[FILTER_STDERR];
     my $stderr_event  = \$self->[EVENT_STDERR];
+    my $is_active     = \$self->[IS_ACTIVE];
 
     $poe_kernel->state
-      ( $self->[STATE_STDERR] = $self . ' select stderr',
+      ( $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
         sub {
           # prevents SEGV
           0 && CRIMSON_SCOPE_HACK('<');
@@ -492,7 +540,13 @@ sub _define_stderr_reader {
           }
           else {
             $$error_event and
-              $k->call( $me, $$error_event, 'read', ($!+0), $!, $unique_id );
+              $k->call( $me, $$error_event,
+                        'read', ($!+0), $!, $unique_id, 'STDERR'
+                      );
+            unless (--$$is_active) {
+              $k->call( $me, $$close_event, $unique_id )
+                if defined $$close_event;
+            }
             $k->select_read($handle);
           }
         }
@@ -540,7 +594,9 @@ sub event {
     }
     elsif ($name eq 'ErrorEvent') {
       $self->[ERROR_EVENT] = $event;
-      $redefine_stdin = $redefine_stdout = $redefine_stderr = 1;
+    }
+    elsif ($name eq 'CloseEvent') {
+      $self->[CLOSE_EVENT] = $event;
     }
     else {
       carp "ignoring unknown Run parameter '$name'";
@@ -559,20 +615,28 @@ sub DESTROY {
   my $self = shift;
 
   # Turn off the STDIN thing.
-  $poe_kernel->select($self->[HANDLE_STDIN]);
+  if ($self->[HANDLE_STDIN]) {
+    $poe_kernel->select($self->[HANDLE_STDIN]);
+    $self->[HANDLE_STDIN] = undef;
+  }
   if ($self->[STATE_STDIN]) {
     $poe_kernel->state($self->[STATE_STDIN]);
     $self->[STATE_STDIN] = undef;
   }
 
-  $poe_kernel->select($self->[HANDLE_STDOUT]);
+  if ($self->[HANDLE_STDOUT]) {
+    $poe_kernel->select($self->[HANDLE_STDOUT]);
+    $self->[HANDLE_STDOUT] = undef;
+  }
   if ($self->[STATE_STDOUT]) {
     $poe_kernel->state($self->[STATE_STDOUT]);
     $self->[STATE_STDOUT] = undef;
   }
 
-  $poe_kernel->select($self->[HANDLE_STDERR])
-    if defined $self->[HANDLE_STDERR];
+  if ($self->[HANDLE_STDERR]) {
+    $poe_kernel->select($self->[HANDLE_STDERR]);
+    $self->[HANDLE_STDERR] = undef;
+  }
   if ($self->[STATE_STDERR]) {
     $poe_kernel->state($self->[STATE_STDERR]);
     $self->[STATE_STDERR] = undef;
@@ -657,11 +721,12 @@ POE::Wheel::Run - event driven fork/exec with added value
   $program = [ '/usr/bin/cat', '-' ];
 
   $wheel = POE::Wheel::Run->new(
-    Program    => $program,
-    Priority   => +5,                 # Adjust priority.  May need to be root.
-    User       => getpwnam('nobody'), # Adjust UID. May need to be root.
-    Group      => getgrnam('nobody'), # Adjust GID. May need to be root.
-    ErrorEvent => 'oops',             # Event to emit on errors.
+    Program     => $program,
+    Priority    => +5,                 # Adjust priority.  May need to be root.
+    User        => getpwnam('nobody'), # Adjust UID. May need to be root.
+    Group       => getgrnam('nobody'), # Adjust GID. May need to be root.
+    ErrorEvent  => 'oops',             # Event to emit on errors.
+    CloseEvent  => 'child_closed',     # Child closed all output.
 
     StdinEvent  => 'stdin',  # Event to emit when stdin is flushed to child.
     StdoutEvent => 'stdout', # Event to emit with child stdout information.
@@ -712,6 +777,8 @@ process.  It may either be 'pipe' (the default), or 'pty'.
 
 Pty conduits require the IO::Pty module.
 
+=item CloseEvent
+
 =item ErrorEvent
 
 =item StdinEvent
@@ -720,8 +787,15 @@ Pty conduits require the IO::Pty module.
 
 =item StderrEvent
 
+C<CloseEvent> contains the name of an event to emit when the child
+process closes all its output handles.  This is a consistent
+notification that the child will not be sending any more output.  It
+does not, however, signal that the client process has stopped
+accepting input.
+
 C<ErrorEvent> contains the name of an event to emit if something
-fails.  It's optional, and if omitted, it won't emit any errors.
+fails.  It is optional.  If omitted, the wheel will not notify its
+session if any errors occur.
 
 Wheel::Run requires at least one of the following three events:
 
@@ -741,12 +815,16 @@ STDOUT or STDERR handles, respectively.
 
 =item StderrFilter
 
-C<Filter> contains a reference to a POE::Filter class that describes
-how the child process performs input and output.  C<Filter> will be
-used to describe the child's stdin, stdout and stderr.
+C<Filter> contains an instance of a POE::Filter subclass.  The filter
+describes how the child process performs input and output.  C<Filter>
+will be used to describe the child's stdin, stdout and stderr.
+C<Filter> is optional.  If left blank, it will default to an instance
+of C<POE::Filter::Line->new(Literal => "\n");>
 
 C<StdinFilter>, C<StdoutFilter> and C<StderrFilter> can be used
-instead of C<Filter> to set different filters for each handle.
+instead of or in addition to C<Filter>.  They will override the
+default filter's selection in situations where a process' input and
+output are in different formats.
 
 =item Group
 
@@ -837,13 +915,35 @@ destroyed properly when children exit.
 Sends a signal to the child process.  It's useful for processes which
 tend to be reluctant to exit when their terminals are closed.
 
+=back
+
 =head1 EVENTS AND PARAMETERS
 
 =over 2
 
+=item CloseEvent
+
+CloseEvent contains the name of the event Wheel::Run emits whenever a
+child process has closed all its output handles.  It signifies that
+the child will not be sending more information.  In addition to the
+usual POE parameters, each CloseEvent comes with one of its own:
+
+C<ARG0> contains the wheel's unique ID.  This can be used to keep
+several child processes separate when they're managed by the same
+session.
+
+A sample close event handler:
+
+  sub close_state {
+    my ($heap, $wheel_id) = @_[HEAP, ARG0];
+
+    my $child = delete $heap->{child}->{$wheel_id};
+    print "Child ", $child->PID, " has finished.\n";
+  }
+
 =item ErrorEvent
 
-ErrorEvent contains the name on an event that Wheel::Run emits
+ErrorEvent contains the name of an event that Wheel::Run emits
 whenever an error occurs.  Every error event comes with four
 parameters:
 
@@ -856,6 +956,12 @@ respectively.
 
 C<ARG3> contains the wheel's unique ID.
 
+C<ARG4> contains the name of the child filehandle that has the error.
+It may be "STDIN", "STDOUT", or "STDERR".  The sense of C<ARG0> will
+be the opposite of what you might normally expect for these handles.
+For example, Wheel::Run will report a "read" error on "STDOUT" because
+it tried to read data from that handle.
+
 A sample error event handler:
 
   sub error_state {
@@ -912,9 +1018,6 @@ Filter changing hasn't been implemented yet.  Let the author know if
 it's needed.  Better yet, patch the file based on the code in
 Wheel::ReadWrite.
 
-Wheel::Run generates SIGCHLD.  This may eventually cause Perl to
-segfault.  Bleah.
-
 Priority is a delta; there's no way to set it directly to some value.
 
 User must be specified by UID.  It would be nice to support login
@@ -923,7 +1026,7 @@ names.
 Group must be specified by GID.  It would be nice to support group
 names.
 
-ActiveState Perl is not going to like this module one bit.
+ActiveState Perl doesn't like this module one bit.
 
 =head1 AUTHORS & COPYRIGHTS
 
